#include "idl_otbcannyfilter.h"

#include "otbImageFileReader.h"
#include "otbImageFileWriter.h"
#include "otbImage.h"
#include "itkCastImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkCannyEdgeDetectionImageFilter.h"


ORFEOANDIDL_API void otbcannyfilter(int argc, IDL_VPTR argv[], char *argk)
{
  static char inputFilename[MAX_LENGTH];
  static char outputFilename[MAX_LENGTH];
  
  if (argv[0]->value.str.s != NULL)
    strcpy(inputFilename, argv[0]->value.str.s);
  
  if (argv[1]->value.str.s != NULL)
		strcpy(outputFilename, argv[1]->value.str.s);
  
  float variance = 2;
  typedef unsigned char    CharPixelType;  //  IO
  typedef double           RealPixelType;  //  Operations
  const   unsigned int     Dimension = 2;
  
  typedef otb::Image<CharPixelType, Dimension>                             CharImageType;
  typedef otb::Image<RealPixelType, Dimension>                             RealImageType;
  typedef otb::ImageFileReader< CharImageType>                             ReaderType;
  typedef otb::ImageFileWriter< CharImageType>                             WriterType;
  typedef itk::CastImageFilter< CharImageType, RealImageType>              CastToRealFilterType;
  typedef itk::RescaleIntensityImageFilter<RealImageType, CharImageType >  RescaleFilter;
  typedef itk::CannyEdgeDetectionImageFilter<RealImageType, RealImageType> CannyFilter;
  
  //Setting the IO
  ReaderType::Pointer           reader      = ReaderType::New();
  WriterType::Pointer           writer      = WriterType::New();
  CastToRealFilterType::Pointer toReal      = CastToRealFilterType::New();
  RescaleFilter::Pointer        rescale     = RescaleFilter::New();
  //Setting the ITK pipeline filter
  CannyFilter::Pointer          cannyFilter = CannyFilter::New();
  
  reader->SetFileName( inputFilename  );
  writer->SetFileName( outputFilename );
  
  //The output of an edge filter is 0 or 1
  rescale->SetOutputMinimum(   0 );
  rescale->SetOutputMaximum( 255 );
  
  toReal->SetInput( reader->GetOutput() );
  
  cannyFilter->SetInput( toReal->GetOutput() );
  cannyFilter->SetVariance( variance );
  rescale->SetInput( cannyFilter->GetOutput() );
  writer->SetInput( rescale->GetOutput() );
  
  writer->Update();
}

