#include "idl_otbextractalignment.h"

#include "otbImage.h"
#include "otbImageFileReader.h"
#include "otbImageFileWriter.h"
#include "otbDrawPathFilter.h"
#include "otbImageToPathListAlignFilter.h"
#include "itkPolyLineParametricPath.h"

ORFEOANDIDL_API void otbextractalignment(int argc, IDL_VPTR argv[], char *argk)
{
  if(argc!=3)
    {
      std::cout << "Usage : " << argv[0] << " inputImage outputImage epsilon" << std::endl;
      return ;
    }
  
  static char inputFilename[MAX_LENGTH];
  static char outputFilename[MAX_LENGTH];
  
  if (argv[0]->value.str.s != NULL)
    strcpy(inputFilename, argv[0]->value.str.s);

  if (argv[1]->value.str.s != NULL)
    strcpy(outputFilename, argv[1]->value.str.s);
  
  IDL_VPTR idlEpsilon = IDL_CvtDbl(1, &argv[2]);  
  int epsilon = (int)idlEpsilon->value.d;
  
  typedef unsigned short InputPixelType;
  typedef unsigned short OutputPixelType;
  const   unsigned int   Dimension = 2;
  
  typedef otb::Image< InputPixelType,  Dimension >                         InputImageType;
  typedef otb::Image< OutputPixelType, Dimension >                         OutputImageType;
  typedef otb::ImageFileReader< InputImageType  >                          ReaderType;
  typedef otb::ImageFileWriter< OutputImageType >                          WriterType;
  typedef itk::PolyLineParametricPath< Dimension >                         PathType;
  typedef otb::ImageToPathListAlignFilter<InputImageType,PathType>         ListAlignFilterType;
  typedef otb::DrawPathFilter< InputImageType, PathType, OutputImageType > DrawPathFilterType;
  typedef ListAlignFilterType::OutputPathListType                          ListType;

  ReaderType::Pointer          reader      = ReaderType::New();
  WriterType::Pointer          writer      = WriterType::New();
  ListAlignFilterType::Pointer alignFilter = ListAlignFilterType::New();

  reader->SetFileName( inputFilename  );
  writer->SetFileName( outputFilename  );
  
  reader->Update();
  
  alignFilter->SetInput( reader->GetOutput() );
  alignFilter->SetEps( epsilon );
  alignFilter->Update();
  
  ListType*               pathList        = alignFilter->GetOutput(); 
  ListType::Iterator      listIt          = pathList->Begin();
  InputImageType::Pointer backgroundImage = reader->GetOutput();

  while( listIt != pathList->End())
    {
      
      DrawPathFilterType::Pointer drawPathFilter = DrawPathFilterType::New();
      drawPathFilter->SetImageInput( backgroundImage );
      drawPathFilter->SetPathInput( listIt.Get() );
      
      drawPathFilter->SetValue( itk::NumericTraits<OutputPixelType>::max() );
      drawPathFilter->Update();
      
      backgroundImage = drawPathFilter->GetOutput();
      
      ++listIt;
      
    }
  writer->SetInput( backgroundImage );
  
  writer->Update(); 
}

