#include "idl_otbextractroad.h"

#include "otbVectorImage.h"
#include "otbImage.h"
#include "otbPolyLineParametricPathWithValue.h"
#include "otbRoadExtractionFilter.h"
#include "otbDrawPathListFilter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "otbImageFileReader.h"
#include "otbImageFileWriter.h"
#include "itkBinaryBallStructuringElement.h"
#include "itkGrayscaleDilateImageFilter.h"
#include "itkInvertIntensityImageFilter.h"

ORFEOANDIDL_API void otbextractroad(int argc, IDL_VPTR argv[], char *argk )
{
  
  if(argc != 13)
    {
      std::cerr << "Usage: "<< argv[0];
      std:: cerr <<" inputFileName outputFileName firstPixelComponent secondPixelComponent ";
      std::cerr <<"thirdPixelComponent fourthPixelComponent alpha amplitudeThrehsold tolerance ";
      std::cerr <<"angularThreshold firstMeanDistanceThreshold secondMeanDistanceThreshold ";
      std::cerr<<"distanceThreshold"<<std::endl;
      return;
    }
  
  static char inputFilename[MAX_LENGTH];
  static char outputFilename[MAX_LENGTH];
  
  if (argv[0]->value.str.s != NULL)
    strcpy(inputFilename, argv[0]->value.str.s);
  
  if (argv[1]->value.str.s != NULL)
    strcpy(outputFilename, argv[1]->value.str.s);
  
  IDL_VPTR idlFirstPixelComponent         = IDL_CvtDbl(1, &argv[2]);  
  IDL_VPTR idlSecondPixelComponent        = IDL_CvtDbl(1, &argv[3]);  
  IDL_VPTR idlThirdPixelComponent         = IDL_CvtDbl(1, &argv[4]);  
  IDL_VPTR idlFourthPixelComponent        = IDL_CvtDbl(1, &argv[5]); 
  IDL_VPTR idlAplha                       = IDL_CvtDbl(1, &argv[6]);
  IDL_VPTR idlAmplitudeThreshold          = IDL_CvtDbl(1, &argv[7]);
  IDL_VPTR idlTolerance                   = IDL_CvtDbl(1, &argv[8]);
  IDL_VPTR idlAngularThreshold            = IDL_CvtDbl(1, &argv[9]);
  IDL_VPTR idlfirstMeanDistanceThreshold  = IDL_CvtDbl(1, &argv[10]);
  IDL_VPTR idlsecondMeanDistanceThreshold = IDL_CvtDbl(1, &argv[11]);
  IDL_VPTR idldistanceThreshold           = IDL_CvtDbl(1, &argv[12]);
  
  double firstPixelComponent          = (double)idlFirstPixelComponent->value.d;
  double secondPixelComponent         = (double)idlSecondPixelComponent->value.d;
  double thirdPixelComponent          = (double)idlThirdPixelComponent->value.d;
  double fourthPixelComponent         = (double)idlFourthPixelComponent->value.d;
  double alpha                        = (double)idlAplha->value.d;
  double amplitudeThreshold           = (double)idlAmplitudeThreshold->value.d;
  double tolerance                    = (double)idlTolerance->value.d;
  double angularTreshold              = (double)idlAngularThreshold->value.d;
  double firstMeanDistanceThreshold   = (double)idlfirstMeanDistanceThreshold->value.d;
  double secondMeanDistanceThreshold  = (double)idlsecondMeanDistanceThreshold->value.d;
  double distanceThreshold            = (double)idldistanceThreshold->value.d;

  const unsigned int    Dimension = 2;
  typedef double        InputPixelType;
  typedef unsigned char OutputPixelType;

   typedef otb::VectorImage<InputPixelType,Dimension>                        InputVectorImageType;
   typedef otb::Image<InputPixelType,Dimension>                              InputImageType;
   typedef otb::Image<OutputPixelType,Dimension>                             OutputImageType;
   typedef otb::PolyLineParametricPathWithValue<InputPixelType,Dimension>    PathType;
   typedef otb::RoadExtractionFilter<InputVectorImageType, PathType>         RoadExtractionFilterType;
   typedef otb::DrawPathListFilter<InputImageType, PathType, InputImageType> DrawPathFilterType;
   typedef itk::RescaleIntensityImageFilter<InputImageType, OutputImageType> RescalerType; 
   typedef otb::ImageFileReader<InputVectorImageType>                        ReaderType; 
   typedef otb::ImageFileWriter<OutputImageType>                             WriterType;


   ReaderType::Pointer               reader                = ReaderType::New();
   RoadExtractionFilterType::Pointer roadExtractionFilter  = RoadExtractionFilterType::New();
   DrawPathFilterType::Pointer       drawingFilter         = DrawPathFilterType::New();
   RescalerType::Pointer             rescaleFilter         = RescalerType::New();
   WriterType::Pointer               writer                = WriterType::New();

   reader->SetFileName(inputFilename);
      
   InputVectorImageType::PixelType ReferencePixel;
   ReferencePixel.SetSize(4);
   ReferencePixel.SetElement(0,firstPixelComponent);
   ReferencePixel.SetElement(1,secondPixelComponent);
   ReferencePixel.SetElement(2,thirdPixelComponent);
   ReferencePixel.SetElement(3,fourthPixelComponent);
   
   roadExtractionFilter->SetReferencePixel(             ReferencePixel);
   roadExtractionFilter->SetAlpha(                      alpha);
   roadExtractionFilter->SetAmplitudeThreshold(         amplitudeThreshold);
   roadExtractionFilter->SetTolerance(                  tolerance);
   roadExtractionFilter->SetMaxAngle(                   angularTreshold);
   roadExtractionFilter->SetAngularThreshold(           angularTreshold);
   roadExtractionFilter->SetFirstMeanDistanceThreshold( firstMeanDistanceThreshold);
   roadExtractionFilter->SetSecondMeanDistanceThreshold(secondMeanDistanceThreshold);
   roadExtractionFilter->SetDistanceThreshold(          distanceThreshold);
     
   reader->GenerateOutputInformation();
   InputImageType::Pointer blackBackground = InputImageType::New();
   blackBackground->SetRegions(reader->GetOutput()->GetLargestPossibleRegion());
   blackBackground->Allocate();
   blackBackground->FillBuffer(0);
   
   drawingFilter->UseInternalPathValueOn();
   
   rescaleFilter->SetOutputMinimum(itk::NumericTraits< OutputPixelType >::min());
   rescaleFilter->SetOutputMaximum(itk::NumericTraits< OutputPixelType >::max());
   
   roadExtractionFilter->SetInput(reader->GetOutput());
   drawingFilter->SetInput(blackBackground);
   drawingFilter->SetInputPath(roadExtractionFilter->GetOutput());
   rescaleFilter->SetInput(drawingFilter->GetOutput());
   
   rescaleFilter->Update();      
   
   // output image enhancement
   typedef itk::BinaryBallStructuringElement<OutputPixelType,Dimension>                            StructuringElementType;
   typedef itk::GrayscaleDilateImageFilter<OutputImageType,OutputImageType,StructuringElementType> DilateFilterType;
   typedef itk::InvertIntensityImageFilter<OutputImageType,OutputImageType>                        InvertFilterType;

   StructuringElementType se;
   se.SetRadius(1);
   se.CreateStructuringElement();
   
   DilateFilterType::Pointer dilater = DilateFilterType::New();

   dilater->SetInput(rescaleFilter->GetOutput());
   dilater->SetKernel(se);
   
   InvertFilterType::Pointer invertFilter = InvertFilterType::New();
   invertFilter->SetInput(dilater->GetOutput());
   
   writer->SetFileName(outputFilename);
   writer->SetInput(invertFilter->GetOutput());
   writer->Update();
   
}
