#include "idl_otbfrostfilter.h"

#include "itkImportImageFilter.h"
#include "otbImage.h"
#include "otbFrostImageFilter.h"

ORFEOANDIDL_API IDL_VPTR otbfrostfilter(int argc, IDL_VPTR argv[], char *argk)
{
  // Variable IDL de sortie :
  IDL_VPTR dst;

  // Wrapper :
  IDL_VPTR src        = IDL_CvtDbl(1, &argv[0]);  
  IDL_VPTR idlRadiusX = IDL_CvtDbl(1, &argv[1]);  
  IDL_VPTR idlRadiusY = IDL_CvtDbl(1, &argv[2]);  
  IDL_VPTR idlDeramp  = IDL_CvtDbl(1, &argv[3]);  
  
  typedef double InputPixelType;
  const unsigned int Dimension = 2;
  
  // Parametres d'entree :
  unsigned int RadiusX = (unsigned int)idlRadiusX->value.d;
  unsigned int RadiusY = (unsigned int)idlRadiusY->value.d;
  double Deramp = idlDeramp->value.d;
  
  // Variable IDL :
  double *temp = (double *)malloc(src->value.arr->n_elts*sizeof(double));
  memcpy((char *)temp, (char *)src->value.arr->data, src->value.arr->arr_len);
  
  // Premier octet de la variable IDL :
  char *firstByte = (char *)IDL_VarMakeTempFromTemplate(src, src->type, NULL, &dst, true);  
      
  // Transfert IDL->ORFEO :
  typedef itk::ImportImageFilter<InputPixelType, Dimension> ImportFilterType;
  ImportFilterType::Pointer importFilter = ImportFilterType::New(); 
  ImportFilterType::SizeType  size;
  
  size[0]  = src->value.arr->dim[0];  // size along X
  size[1]  = src->value.arr->dim[1];  // size along Y
  
  ImportFilterType::IndexType start;
  start.Fill( 0 );
  
  ImportFilterType::RegionType region;
  region.SetIndex( start );
  region.SetSize(  size  );
  
  importFilter->SetRegion( region );
  
  // Copie :
  importFilter->SetImportPointer(temp, src->value.arr->n_elts, true);
  
  // Definition ORFEO :
  typedef double OutputPixelType;
  typedef otb::Image< InputPixelType,  Dimension > InputImageType;
  typedef otb::Image< OutputPixelType, Dimension > OutputImageType;
  
  // Creation d'un filtre de LEE :
  typedef otb::FrostImageFilter< InputImageType,OutputImageType >   FilterType;
  
  // Definition des parametres du filtre de LEE :
  FilterType::SizeType Radius;
  Radius[0]= RadiusX;
  Radius[1]= RadiusY;
  FilterType::Pointer filtreFrost = FilterType::New();
  
  // AJOUTER ICI LE CODE SPECIFIQUE POUR UTILISER LE FILTRE DE LEE SUR LA VARIABLE importFilter :
  
  filtreFrost->SetInput((OutputImageType *)importFilter->GetOutput());
  
  // Definition des parametres :
  filtreFrost->SetRadius( Radius );
  filtreFrost->SetDeramp( Deramp );
  
  // Mise a jour du filtre :
  filtreFrost->Update();
  
  // Recuperation des donnees filtrees et transfert dans la variable IDL de sortie :
  double* pData = (double *)filtreFrost->GetOutput()->GetBufferPointer();
  memcpy((char *)firstByte, (char *)pData, src->value.arr->arr_len);
  
  
  // Renvoi de la variable IDL vers le monde IDL :
  return dst;
}
