#include "idl_otbsvmimagemodelestimator.h"

#include "otbImage.h"
#include "otbVectorImage.h"
#include "otbSVMImageModelEstimator.h"
#include "otbImageFileReader.h"

ORFEOANDIDL_API void otbsvmimagemodelestimator(int argc, IDL_VPTR argv[], char *argk)
{
  static char inputImageFileName[MAX_LENGTH];
  static char trainingImageFilename[MAX_LENGTH];
  static char outputModelFilename[MAX_LENGTH];
  
  if (argv[0]->value.str.s != NULL)
    strcpy(inputImageFileName, argv[0]->value.str.s);
  
  if (argv[1]->value.str.s != NULL)
    strcpy(trainingImageFilename, argv[1]->value.str.s);
  
  if (argv[2]->value.str.s != NULL)
    strcpy(outputModelFilename, argv[2]->value.str.s);
  
  typedef unsigned char      InputPixelType;
  const   unsigned int       Dimension = 2;
  
  typedef otb::VectorImage<InputPixelType, Dimension>                    InputImageType;
  typedef otb::Image<InputPixelType, Dimension>                          TrainingImageType;
  typedef otb::SVMImageModelEstimator<InputImageType, TrainingImageType> EstimatorType;
  typedef otb::ImageFileReader< InputImageType >                         InputReaderType;
  typedef otb::ImageFileReader< TrainingImageType >                      TrainingReaderType;
  
  InputReaderType::Pointer inputReader = InputReaderType::New();
  TrainingReaderType::Pointer trainingReader = TrainingReaderType::New();
  
  inputReader->SetFileName( inputImageFileName );
  trainingReader->SetFileName( trainingImageFilename );
  
  inputReader->Update();
  trainingReader->Update();
  
  EstimatorType::Pointer svmEstimator = EstimatorType::New();
  
  svmEstimator->SetInputImage( inputReader->GetOutput() );
  svmEstimator->SetTrainingImage( trainingReader->GetOutput() );
  svmEstimator->SetNumberOfClasses( 2 );
  svmEstimator->Update();
  svmEstimator->SaveModel(outputModelFilename);
}
