#include "idl_otbtouziedgedetector.h"

#include "otbImage.h"
#include "otbImageFileWriter.h"
#include "otbImageFileReader.h"
#include "otbTouziEdgeDetectorImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"

ORFEOANDIDL_API void otbtouziedgedetector(int argc, IDL_VPTR argv[], char *argk)
{
  if( argc != 3 )
    {
      std::cerr << "Usage: " << argv[0] << " inputImageFile ";
      std::cerr << " outputEdgesImageFile outputDirectionsImageFile radius" << std::endl;  
      return;
    }
  
  static char inputFilename[MAX_LENGTH];
  static char outputFilename[MAX_LENGTH];
  
  if (argv[0]->value.str.s != NULL)
    strcpy(inputFilename, argv[0]->value.str.s);
  
  if (argv[1]->value.str.s != NULL)
    strcpy(outputFilename, argv[1]->value.str.s);
  
  IDL_VPTR idlRadius = IDL_CvtDbl(1, &argv[2]);  
  int radius = (int)idlRadius->value.d;
  
 
  typedef  float  InternalPixelType;
  typedef  unsigned char  OutputPixelType;
  
  typedef otb::Image< InternalPixelType,  2 >                                        InternalImageType;
  typedef otb::Image< OutputPixelType,  2 >                                          OutputImageType;
  typedef otb::TouziEdgeDetectorImageFilter< InternalImageType, InternalImageType >  FilterType;
  typedef otb::ImageFileReader< InternalImageType >                                  ReaderType;
  typedef otb::ImageFileWriter< OutputImageType >                                    WriterType;
  typedef itk::RescaleIntensityImageFilter< InternalImageType, OutputImageType >     RescalerType; 

  ReaderType::Pointer   reader   = ReaderType::New();
  FilterType::Pointer   filter   = FilterType::New();
  RescalerType::Pointer rescaler = RescalerType::New();
  WriterType::Pointer   writer   = WriterType::New();
  
  reader->SetFileName( inputFilename );

  rescaler->SetOutputMinimum( itk::NumericTraits< OutputPixelType >::min());
  rescaler->SetOutputMaximum( itk::NumericTraits< OutputPixelType >::max());
  
  filter->SetInput( reader->GetOutput() );
  rescaler->SetInput( filter->GetOutput() );
  writer->SetInput( rescaler->GetOutput() );

  FilterType::SizeType Radius;
  Radius[0]= radius;
  Radius[1]= radius;

  filter->SetRadius( Radius );
 
  filter->Update();
 
  writer->SetFileName( outputFilename );
  writer->Update();

  writer->SetFileName( outputFilename );
  
  rescaler->SetInput( filter->GetOutputDirection() );
  writer->SetInput( rescaler->GetOutput() );
  writer->Update();

  return;
}

